/******************************************************************
 * CopyPolicy: GNU General Public License version 2
 * Copyright 2004 Peter Jones <pjones@redhat.com>
 * 
 * macros to generate scsi commands.
 *
 ******************************************************************/

#ifndef _SCSI_CMDS_H
#define _SCSI_CMDS_H 1
#include <scsi/scsi.h>

/* from the MMC3 spec, rewritten as seperate macros */
#define LBA_to_M(lba) (lba>=-150?((lba+150)/(60*75)):((lba+450150)/(60*75)))
#define LBA_to_S(lba) (lba>=-150?((lba+150)/75):((lba+450150)/75))
#define LBA_to_F(lba) (lba>=-150?(lba+150):(lba+450150))

/* Group 1 (10b) command */
#define SCSI_TWELVE_BYTE(a,b,c,d,e,f,g,h,i,j,k,l) ((char []) {a,b,c,d,e,f,g,h,i,j,k,l})
#define SCSI_READ_12(fua, a, l) SCSI_TWELVE_BYTE( \
	READ_12,		/* READ_10 */ \
	(fua & 1) << 3,		/* force unit access */ \
	(a >> 24) & 0xff,	/* lba byte 3 */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	(l >> 24) & 0xff,	/* len byte 3 */ \
	(l >> 16) & 0xff,	/* len byte 2 */ \
	(l >> 8) & 0xff,	/* len byte 1 */ \
	l & 0xff,		/* len byte 0 */ \
	0			/* control */ \
)
#define D4_READ_12(a, l) SCSI_TWELVE_BYTE( \
	0xD4,			/* 0xD4 */ \
	0,			/* lun */ \
	0,			/* ? */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	0,			/* ? */ \
	0,			/* ? */ \
	l & 0xff,		/* len byte 0 */ \
	0, 0			/* ? */ \
)
#define D8_READ_12(a, l) SCSI_TWELVE_BYTE( \
	0xD8,			/* 0xD4 */ \
	0,			/* lun */ \
	0,			/* ? */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	0,			/* ? */ \
	0,			/* ? */ \
	l & 0xff,		/* len byte 0 */ \
	0, 0			/* ? */ \
)
#define READ_CD_12(a, l) SCSI_TWELVE_BYTE( \
	0xBE,			/* 0xD4 */ \
	0,			/* ? */ \
	(a >> 24) & 0xff,	/* lba byte 3 */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	(l >> 16) & 0xff,	/* len byte 2 */ \
	(l >> 8) & 0xff,	/* len byte 1 */ \
	l & 0xff,		/* len byte 0 */ \
	10,			/* ecc */ \
	0, 0			/* ? */ \
)
#define READ_CD_MSF_12(a, l) SCSI_TWELVE_BYTE( \
	0xB9,			/* 0xD4 */ \
	0,			/* ? */ \
	0,			/* ? */ \
	LBA_to_M((a)),		/* start M */ \
	LBA_to_S((a)),		/* start S */ \
	LBA_to_F((a)),		/* start F */ \
	LBA_to_M((a)+(l)),	/* start M */ \
	LBA_to_S((a)+(l)),	/* start S */ \
	LBA_to_F((a)+(l)),	/* start F */ \
	10,			/* ecc */ \
	0, 0			/* ? */ \
)

#define SCSI_TEN_BYTE(a,b,c,d,e,f,g,h,i,j) ((char []) {a,b,c,d,e,f,g,h,i,j})
#define SCSI_MODE_SENSE_10(page, size) SCSI_TEN_BYTE( \
	MODE_SENSE_10,	/* MODE_SENSE */ \
	0x00,		/* reserved */ \
	page & 0x3F,		/* page */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* MSB (0) */ \
	size,		/* sizeof(modesense - SG_OFF) */ \
	0		/* reserved */ \
)
#define SCSI_MODE_SELECT_10 SCSI_TEN_BYTE( \
	MODE_SELECT_10,	/* MODE_SELECT */ \
	0x10,		/* no save page */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	12,		/* sizeof(mode) */ \
	0		/* reserved */ \
)
#define SCSI_READ_TOC(track_number) SCSI_TEN_BYTE( \
	READ_TOC,	/* READ_TOC */ \
	0,		/* MSF format */ \
	0,		\
	0,		\
	0,		\
	0,		\
	track_number,	/* start track */ \
	0,		/* len msb */ \
	12,		/* len lsb */ \
	0		/* flags */ \
)
/* a contribution from Boris for IMS cdd 522 */
/* check this for ACER/Creative/Foo 525,620E,622E, etc? */
#define CDD522_READ_TOC(track_number) SCSI_TEN_BYTE( \
	0xE5,		/* CDD522_READ_TOC */ \
	0, 0, 0, 0,	/* res */ \
	track_number,	/* start track */ \
	0, 0, 0, 0	/* ? */ \
)
#define SCSI_READ_10(fua, a, l) SCSI_TEN_BYTE( \
	READ_10,		/* READ_10 */ \
	(fua?8:0),		/* force unit access */ \
	(a >> 24) & 0xff,	/* lba byte 3 */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	(l >> 8) & 0xff,	/* len byte 1 */ \
	l & 0xff,		/* len byte 0 */ \
	0			/* control */ \
)
#define D4_READ_10(a, l) SCSI_TEN_BYTE( \
	0xD4,			/* 0xD4 */ \
	0,			/* ? */ \
	0,			/* ? */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	0,			/* ? */ \
	l & 0xff,		/* len byte 0 */ \
	0			/* control */ \
)
#define D5_READ_10(a, l) SCSI_TEN_BYTE( \
	0xD5,			/* 0xD5 */ \
	0,			/* lun */ \
	0,			/* ? */ \
	(a >> 16) & 0xff,	/* lba byte 2 */ \
	(a >> 8) & 0xff,	/* lba byte 1 */ \
	a & 0xff,		/* lba byte 0 */ \
	0,			/* reserved */ \
	0,			/* ? */ \
	l & 0xff,		/* len byte 0 */ \
	0			/* control */ \
)


#define SCSI_SIX_BYTE(a,b,c,d,e,f) ((char []) {a,b,c,d,e,f})
#define SCSI_MODE_SENSE_6(page, size) SCSI_SIX_BYTE( \
	MODE_SENSE,	/* MODE_SENSE */ \
	0x00,		/* return block descriptor/lun */ \
	page & 0x3F,		/* page */ \
	0,		/* reserved */ \
	size,		/* sizeof(modesense - SG_OFF) */ \
	0		/* control */  \
)
#define SCSI_MODE_SELECT_6 SCSI_SIX_BYTE( \
	MODE_SELECT,	/* MODE_SELECT */ \
	0x10,		/* no save page */ \
	0,		/* reserved */ \
	0,		/* reserved */ \
	12,		/* sizeof(mode) */ \
	0		/* reserved */ \
)
#define SCSI_INQUIRY_6(len) SCSI_SIX_BYTE( \
	INQUIRY,	/* INQUIRY */ \
	0, 0, 0,	/* ? */ \
	len, 0		/* len, ? */ \
)
#define SCSI_TEST_UNIT_READY_6 SCSI_SIX_BYTE( \
	TEST_UNIT_READY,/* TEST_UNIT_READY */ \
	0, 0, 0, 0,	/* reserved */ \
	0		/* control */ \
)
#endif
